/*
 * rtp2wav.cxx
 *
 * Open Phone Abstraction Library (OPAL)
 * Formally known as the Open H323 project.
 *
 * Copyright (c) 2001 Equivalence Pty. Ltd.
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 *
 * The Original Code is Open Phone Abstraction Library.
 *
 * The Initial Developer of the Original Code is Equivalence Pty. Ltd.
 *
 * Contributor(s): ______________________________________.
 *
 * $Log: rtp2wav.cxx,v $
 * Revision 1.1  2002/05/21 02:42:58  robertj
 * Added class to allow for saving of RTP data to a WAV file.
 *
 */

#include <ptlib.h>

#ifdef __GNUC__
#pragma implementation "rtp2wav.h"
#endif

#include "rtp2wav.h"


#define new PNEW


///////////////////////////////////////////////////////////////////////////////

OpalRtpToWavFile::OpalRtpToWavFile()
#ifdef _MSC_VER
#pragma warning(disable:4355)
#endif
  : receiveHandler(PCREATE_NOTIFIER(ReceivedPacket))
#ifdef _MSC_VER
#pragma warning(default:4355)
#endif
{
  payloadType = RTP_DataFrame::IllegalPayloadType;
}


OpalRtpToWavFile::OpalRtpToWavFile(const PString & filename)
#ifdef _MSC_VER
#pragma warning(disable:4355)
#endif
  :  receiveHandler(PCREATE_NOTIFIER(ReceivedPacket))
#ifdef _MSC_VER
#pragma warning(default:4355)
#endif
{
  SetFilePath(filename);
  payloadType = RTP_DataFrame::IllegalPayloadType;
}


void OpalRtpToWavFile::ReceivedPacket(RTP_DataFrame & frame, INT)
{
  if (payloadType == RTP_DataFrame::IllegalPayloadType) {
    payloadType = frame.GetPayloadType();
    static int SupportedTypes[] = {
      PWAVFile::fmt_uLaw,
      0, 0,
      PWAVFile::fmt_GSM,
      PWAVFile::fmt_VivoG7231,
      0, 0, 0,
      PWAVFile::fmt_ALaw,
      0, 0,
      PWAVFile::fmt_PCM
    };
    if (payloadType < PARRAYSIZE(SupportedTypes) && SupportedTypes[payloadType] != 0) {
      if (!SetFormat(SupportedTypes[payloadType])) {
        PTRACE(1, "rtp2wav\tCould not set WAV file format: " << SupportedTypes[payloadType]);
        return;
      }
      if (!Open(PFile::WriteOnly)) {
        PTRACE(1, "rtp2wav\tCould not open WAV file: " << GetErrorText());
        return;
      }
    }
    else {
      PTRACE(1, "rtp2wav\tUnsupported payload type: " << payloadType);
    }
  }

  if (payloadType != frame.GetPayloadType())
    return;

  if (!IsOpen())
    return;

  PINDEX payloadSize = frame.GetPayloadSize();
  if (payloadSize > 0) {
    if (Write(frame.GetPayloadPtr(), payloadSize)) {
      memcpy(lastFrame.GetPointer(payloadSize), frame.GetPayloadPtr(), payloadSize);
      return;
    }
  }
  else {
    if (Write(lastFrame.GetPointer(), lastFrame.GetSize()))
      return;
  }

  PTRACE(1, "rtp2wav\tError writing to WAV file: " << GetErrorText(PChannel::LastWriteError));
  Close();
}


/////////////////////////////////////////////////////////////////////////////
